@@ -9,6 +9,8 @@
 #ifndef __KEYD_H__
 #define __KEYD_H__
 
+#include <stdint.h>
+
 #define KEYD_SOCKET "keyd.sock"
 
 enum keyd_ops {
@@ -21,7 +23,9 @@ enum keyd_ops {
 	KEYD_CMD_GETFULLKEYID,
 	KEYD_CMD_KEYITER,
 	KEYD_CMD_CLOSE,
-	KEYD_CMD_QUIT
+	KEYD_CMD_QUIT,
+	KEYD_CMD_STATS,
+	KEYD_CMD_LAST			/* Placeholder */
 };
 
 enum keyd_reply {
@@ -29,6 +33,12 @@ enum keyd_reply {
 	KEYD_REPLY_UNKNOWN_CMD = 1
 };
 
-static int keyd_version = 2;
+static uint32_t keyd_version = 2;
+
+struct keyd_stats {
+	time_t started;
+	uint32_t connects;
+	uint32_t command_stats[KEYD_CMD_LAST];
+};
 
 #endif /* __KEYD_H__ */
